#! /usr/bin/perl
#
# Ziel ist es die Cyrus-Daten zweier verschiedener Rechner (auch Architekturen)
# zu synchornisieren.
#
# Wir kümmern uns hier nicht um das Anlegen der Mailbox, das muss anderweitig
# geschehen.
#
# Die eigentlich Synchronisation wird durch das Tool imapsync vorgenommen, z.B.
# so:
#
# imapsync --ssl1 --host1 pu.schlittermann.de --user1 mobst@schlittermann.de
# --password1 gahlij \ --ssl2 --host2 cyrus.schlittermann.de --user2
# mobst@schlittermann.de --password2 gahlij
#
# Es ist also notwendig für jeden zu synchronisierende Mailbox das Passwort zu
# kennen.  Dies lässt sich allerdings in unserm Falle leicht bewerkstelligen,
# da wir die Passworte im Klartext in einer Mysql-Datenbank liegen haben.
# (Theoretisch hat imapsync auch eine Funktion um mit Adminaccount zu Arbeitn,
# allerdings hat das nicht auf Anhieb funktoniert)
#
# Auf der Stardardeingabe (alternativ auf Kommandozeilenargumente) wird eine
# Liste mit zu importierenden Mailboxen erwartet.  Man kann sich einfach eine
# Liste aller eingerichtete# Mailboxen mit
#
# su - cyrus -c '/usr/sbin/ctl_mboxlist -d' | awk '{ print $4}' | sort |uniq 
#
# geben lassen.
#
#  -- <mobst@schlittermann.de>

my $usage = <<'#';
Usage: $ME <options> mailboxlist.txt
    
    --dry-run	    dry run ($opt_dry)
    --skip	    skipp missing mailboxes ($opt_skip)
    --logdir=s	    directory where logfiles will go to ($opt_logdir)
    --maxage=i	    synchronize only mails younger than $(opt_maxage)
    --help	    this help text
    --folders	    only sync folders
    --acls	    fix ACLs
    --shared	    migrage shared mailboxes
#

use strict;
use warnings;
use Smart::Comments;
use feature ':5.10';
use File::Basename;
use DBI;
use Data::Dumper;
use Getopt::Long;
use File::Path;

END {
    my $acl_file = 'cyrus_mbox_acls.txt';
    -f  $acl_file && unlink $acl_file;
}



my $ME = basename $0;

my $dsn =  "DBI:mysql:database=mail;host=pu.schlittermann.de";
my $db_user = "exim";
my $db_pass = undef;

my $imap_from_host  = "pu.schlittermann.de";
my $imap_to_host    = "pop.net.schlittermann.de";

my $opt_dry = 0;
my $opt_skip = 0;
my $opt_logdir = "./imapsync-log/";
my $opt_maxage = 0;
my $opt_folders = 0;
my $opt_acls = 0;
my $opt_shared = 0;

GetOptions(
    "dry-ryn"	    =>	\$opt_dry,
    "skip"	    =>	\$opt_skip,
    "folders"	    =>	\$opt_folders,
    "acls"	    =>	\$opt_acls,
    "shared"	    =>	\$opt_shared,
    "logdir=s"	    =>	\$opt_logdir,
    "maxage=i"	    =>	\$opt_maxage,
) or die eval "\"$usage\"";

if ($opt_acls) {
    system("su - cyrus -c '/usr/sbin/ctl_mboxlist -d' > cyrus_mbox_acls.txt");
    #system('cat cyrus_mbox_acls.txt | sed s/root@[^\t]*/root/ | su - cyrus -c "/usr/sbin/ctl_mboxlist -u"');
    system('cat cyrus_mbox_acls.txt | sed \'s/root@[^\t]*/root/\' | su - cyrus -c "/usr/sbin/ctl_mboxlist -u"');
    
    exit 0;
}

if ($opt_shared) {
    system("./imapsync --syncacls --ssl1 --ssl2 --prefix2 'Shared Folders' --prefix1 'Shared Folders' --sep1 / --sep2 / --host1 $imap_from_host --host2  $imap_to_host     --subscribe --user1 heiko\@schlittermann.de --user2 heiko\@schlittermann.de --authuser1 root --authuser2 root   --password1 x --password2 x --folder 'Shared Folders/domains'");
    exit 0;
}



### connection to mysql account database
my $dbh = DBI->connect($dsn, $db_user, $db_pass, { RaiseError => 1} ) or
    die "$ME: cannot conncet to mysql: $!\n";

### logfiles will go to: $opt_logdir
-d $opt_logdir or mkpath $opt_logdir or die "$ME: cannot create log dir: $!\n";

my %password_of_account_ =  ();

### reading list of mailboxes to import
while (defined (my $mailbox = <>)) {
    chomp($mailbox);
    my ($user, $domain) = split('@', $mailbox);

    # lookup user@domain in accout db and save password
    my $sth = $dbh->prepare("SELECT userPassword FROM users WHERE user = ? AND domain = ?"); 
    $sth->execute($user, $domain);
    my $pass = ($sth->fetchrow_array())[0];
    
    if (not defined $pass) {
	warn "$ME: mailbox $mailbox does not existion on imap server!\n";
	$opt_skip or exit;
	next;
    }

    $password_of_account_{$mailbox} = $pass;
}

### starting imap migration

foreach my $mailbox (keys %password_of_account_) {
    ### processing mailbox: $mailbox
    my $imapsync_options = "";
    $imapsync_options .= " --dry"		    if $opt_dry;
    $imapsync_options .= " --maxage=$opt_maxage"    if $opt_maxage > 0;

    my $logfile = ">> $opt_logdir/$mailbox.log";
    unlink("opt_logdir/$mailbox.log");

    my $cmd = <<EOF;
./imapsync --ssl1 --ssl2 --prefix2 '' --sep2 / --host1 $imap_from_host --host2 $imap_to_host 
    --subscribe --user1 $mailbox --user2 $mailbox --authuser1 root --authuser2 root 
    --password1 x --password2 x 
    --exclude 'Shared Folders' --exclude 'Other Users'
EOF
    $cmd =~ s/\n/ /g;

    if ($opt_folders) {
	# only INBOX
	system("$cmd --syncacls --justfolders --folderrec INBOX $logfile");
	warn "$ME: migration of mailbox $mailbox failed, see $logfile\n" if ($? != 0);

	# now the remaining mailboxes
	system("$cmd --syncacls --justfolders $logfile");
	warn "$ME: migration of mailbox $mailbox failed, see $logfile\n" if ($? != 0);
    } else {
	system("$cmd --expunge2 --delete2 $logfile");
	warn "$ME: migration of mailbox $mailbox failed, see $logfile\n" if ($? != 0);
    }
}

# vim: sw=4 sts=4 aw
